/**
  *  \file client/dialogs/funcview.cc
  *  \brief Hull Function Display
  *
  *  (c) 2001-2015 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/dialogs/funcview.h"
#include "client/widgets/funclist.h"
#include "client/widgets/funcinfo.h"
#include "util/nls.h"
#include "gfx/palette.h"
#include "ui/layout.h"
#include "ui/widgets/keytranslator.h"
#include "ui/widgets/framegroup.h"
#include "ui/widgets/button.h"
#include "ui/widgets/quit.h"

namespace {
    class WHullFunctionView : public UIWindow {
        ObjectHolder h;
        WHullFunctionList& list;
        WHullFunctionInfo& info;
     public:
        WHullFunctionView(WHullFunctionList& list);
        ~WHullFunctionView();

        void init();
        void onScroll();
    };
}

WHullFunctionView::WHullFunctionView(WHullFunctionList& list)
    : UIWindow(_("Ship Functions"), UIWindow::BLUE_WINDOW, COLOR_GRAY),
      list(list),
      info(h.add(new WHullFunctionInfo()))
{
    init();
    onScroll();
}

WHullFunctionView::~WHullFunctionView()
{ }

void
WHullFunctionView::init()
{
    // VBox
    //   UIFrameGroup [HBox]
    //     list
    //     UIScrollbar
    //   info
    //   HBox
    //     UIButton "OK"
    //     UISpacer

    setLayout(UIVBoxLayout::instance5);

    UIFrameGroup& group1 = h.add(new UIFrameGroup(UIHBoxLayout::instance0));
    group1.add(list);
    group1.add(h.add(new UIScrollbar(list)));

    UIGroup& group2 = h.add(new UIGroup(UIHBoxLayout::instance5));
    group2.add(h.add(new UIButton(0, _("OK"), SDLK_RETURN)).addStop());
    group2.add(h.add(new UISpacer()));

    add(group1);
    add(info);
    add(group2);
    add(h.add(new UIKeyTranslator()).addStop(' ', 0).addStop(SDLK_ESCAPE, 0));
    add(h.add(new UIQuit(0)));

    pack();

    list.sig_moved.add(this, &WHullFunctionView::onScroll);
}

void
WHullFunctionView::onScroll()
{
    info.setFunction(list.getSelectedFunction());
}

/** Display list of hull functions for a ship query. This executes the
    query and displays the result; if the query returns no result, it
    does nothing.
    \param univ Universe to work with
    \param query Ship query */
void
showHullFunctions(const GUniverse& univ, const GShipQuery& query)
{
    /* Initialize list */
    WHullFunctionList list;
    list.setQuery(univ, query);
    if (list.getSize() == 0)
        return;

    /* List was successfully built, so make a dialog */
    WHullFunctionView window(list);
    ui_root->centerWidget(window);
    ui_root->runModal(window);
}
