/**
  *  \file client/dialogs/gotoxydlg.cc
  *  \brief Dialog: Go To X/Y
  *
  *  (c) 2001-2015 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/dialogs/gotoxydlg.h"
#include "ui/window.h"
#include "ui/msgbox.h"
#include "util/nls.h"
#include "ui/layout.h"
#include "game/coord.h"
#include "gfx/palette.h"
#include "ui/widgets/statictext.h"
#include "ui/widgets/inputline.h"
#include "ui/widgets/button.h"
#include "ui/uiglobal.h"
#include "client/widgets/helpwidget.h"
#include "ui/widgets/quit.h"

namespace {
    class WGoToXYDialog : public UIWindow {
        ObjectHolder h;
        GPoint& result;
        UIInputLine* input;
     public:
        WGoToXYDialog(GPoint& result);
        void init();
        bool handleEvent(const UIEvent& event, bool second_pass);
    };
}

WGoToXYDialog::WGoToXYDialog(GPoint& result)
    : UIWindow(_("Go To X/Y"), UIWindow::BLUE_WINDOW, COLOR_GRAY),
      result(result),
      input(0)
{
    init();
}

void
WGoToXYDialog::init()
{
    // UIWindow
    //   UIMultilineStatic
    //   UIInputLine
    //   UIHBox
    //     UIButton "OK"
    //     UIButton "Cancel"
    //     UISpacer
    setLayout(UIVBoxLayout::instance5);

    add(h.add(new UIStaticText(_("Enter X,Y coordinates or sector number:"), UIStaticText::StaticNormal)));

    input = new UIInputLine(100, 8, 0, UIInputLine::ilf_Framed, 0);
    add(h.add(input));

    UIGroup& g = h.add(new UIGroup(UIHBoxLayout::instance5));
    add(g);
    g.add(h.add(new UIButton(_("OK"), SDLK_RETURN)));
    g.add(h.add(new UIButton(_("Cancel"), SDLK_ESCAPE)));
    g.add(h.add(new UISpacer()));
    g.add(h.add(new UIButton("H", 'h')));
    g.add(h.add(new WHelpWidget("pcc2:gotoxy")));
    g.add(h.add(new UIQuit(0)));

    pack();
}

bool
WGoToXYDialog::handleEvent(const UIEvent& event, bool second_pass)
{
    if (event.type == ev_Key) {
        switch (event.key.code) {
         case SDLK_RETURN:
            if (input->getText().size() == 0) {
                // ignore
            } else if (result.parseCoordinates(input->getText()) || result.parseSectorNumber(input->getText())) {
                // success
                stop(1);
            } else {
                // failure
                messageBox(_("Invalid input."), _("Go To X/Y"));
            }
            return true;

         case SDLK_ESCAPE:
            // Cancel
            stop(0);
            return true;
        }
    }
    return UIWindow::handleEvent(event, second_pass);
}

/** Perform "Go To X/Y" dialog. This dialog asks the user for a coordinate.
    \param out [out] Returns the coordinate if the user entered one
    \retval true when user entered a coordinate
    \retval false when user cancelled the dialog */
bool
doGoToXYDialog(GPoint& out)
{
    WGoToXYDialog dlg(out);
    ui_root->centerWidget(dlg);
    return ui_root->runModal(dlg);
}
