/**
  *  \file client/widgets/allylist.h
  *  \brief Alliance Status Widget
  *
  *  (c) 2001-2015 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_CLIENT_WIDGETS_ALLYLIST_H
#define PCC_V2_CLIENT_WIDGETS_ALLYLIST_H

#include "ui/ui.h"

class GAlliances;

/** Alliance Status Widget.
    Displays a list of players, and the alliance status for each.
    - red if no offer
    - yellow of either side offers
    - green if alliance is established
    - bright red if an enemy order is given

    The player who owns the result file is displayed but cannot be selected.

    User must arrange for WAllyList::drawWidget() to be called if the GAlliances object changes. */
class WAllyList : public UIWidget {
 public:
    WAllyList(int id, int player, GAlliances& editAllies);

    void getLayoutInfo(LayoutInfo&);
    void drawContent(GfxCanvas& can);
    bool handleEvent(const UIEvent& event, bool second_pass);
    void onStateChange(int state, bool enable);

    int getPosition() const;
    void setPosition(int newPos);
    void changePosition(int delta);
    void toggleCurrent();

    Signal1<int> sig_player_change;

 private:
    bool mouseDown;
    int player;
    int position;
    int rightWidth;
    GAlliances& editAllies;
};

#endif
