/**
  *  \file game/msgglobal.cc
  *  \brief Message Parser - Global Instance
  *
  *  This defines the global message parser instance, and hooks it
  *  into the module initialisation.
  *
  *  (c) 2001-2015 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "game/msgglobal.h"
#include "game/gamedirs.h"
#include "game/global.h"
#include "game/inbox.h"
#include "game/msg.h"
#include "game/msgparse.h"
#include "game/univ.h"
#include "io/streams.h"
#include "util/modinit.h"
#include "game/alliance.h"

static GMessageParser global_message_parser;

static void
mergeAllies(GAlliances& allies, const GMessageInformation& mi)
{
    for (GMessageInformation::iterator i = mi.begin(), e = mi.end(); i != e; ++i) {
        if (GMessageAllianceValue* av = dynamic_cast<GMessageAllianceValue*>(*i)) {
            if (GAllianceOffer* pOffer = allies.getMutableOffer(allies.find(av->getIndex()))) {
                pOffer->merge(av->getValue());
            }
        }
    }
}

/** Parse messages.
    \param inbox  [in] Message box containing messages to parse
    \param trn    [in/out] Turn will be updated with new information */
void
parseMessages(GInbox& inbox, GGameTurn& trn)
{
    for (int i = 0, e = inbox.getCount(); i < e; ++i) {
        ptr_vector<GMessageInformation> mi;
        global_message_parser.parseMessage(inbox.getText(i), inbox.getReceiver(i), trn.getTurnNumber(), mi);
        for (uint_t j = 0; j < mi.size(); ++j) {
            if (mi[j]->getObjectType() == MessageAlliance) {
                mergeAllies(trn.getAlliances(inbox.getReceiver(i)), *mi[j]);
            } else {
                trn.getCurrentUniverse().addMessageInformation(*mi[j]);
            }
        }
    }
}

/** Get global message parser. This one can parse a user's incoming messages. */
GMessageParser&
getGlobalMessageParser()
{
    return global_message_parser;
}

/**************************** Module Interface ***************************/

static void
initMessageParser()
{
    global_message_parser.done();
    Ptr<Stream> in = spec_file_dir->openFileNT("msgparse.ini", Stream::C_READ);
    if (in) {
        global_message_parser.load(*in);
    }
}

static void
doneMessageParser()
{
    global_message_parser.done();
}

extern const ModuleInitializer mod_msgglobal[] = {
    { OnEnterDirectory, initMessageParser },
    { OnLeftDirectory | OnEnterNoGame, doneMessageParser },
    { 0, 0 }
};
