/**
  *  \file int/contextio.h
  *  \brief Interpreter: Context I/O
  *
  *  (c) 2001-2015 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_INT_CONTEXTIO_H
#define PCC_V2_INT_CONTEXTIO_H

#include "int/value.h"
#include "cpluslib/ptrvector.h"

/** Tag values for serialized contexts. Those must have their lower 8 bits zero.
    By convention, to avoid collision with standard (non-context) values, they
    also have their highest bit set. */
enum {
    Tag_Ship      = 0x8000,
    Tag_Planet    = 0x8100,
    Tag_Minefield = 0x8200,
    Tag_Ion       = 0x8300,
    Tag_Hull      = 0x8400,
    Tag_Engine    = 0x8500,
    Tag_Beam      = 0x8600,
    Tag_Torpedo   = 0x8700,
    Tag_Launcher  = 0x8800,
    Tag_Frame     = 0x8900,
    Tag_Global    = 0x8A00,
    Tag_Mutex     = 0x8B00,
    Tag_Iterator  = 0x8C00,
    Tag_Player    = 0x8D00
};

class IntContext;

IntContext* loadContext(const IntTagNode& sv, Stream& aux);

void saveContexts(Stream& out, const ptr_vector<IntContext>& contexts);
void loadContexts(Stream& in, uint32_t slots, ptr_vector<IntContext>& contexts);

#endif
