/**
  *  \file tests/testzip.cc
  *  \brief Basic ZipStream / ZipFile access
  *
  *  (c) 2001-2015 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include <iostream>
#include <cstring>
#include "io/streams.h"
#include "io/zipstream.h"
#include "util/except.h"
#include "io/zip.h"

static void cat(Stream& s)
{
    char buf[100];
    while (int i = s.read(buf, sizeof(buf))) {
        std::cout.write(buf, i);
    }
}

int main(int /*argc*/, char** argv)
{
    const char* cmd = *++argv;
    if (!cmd) {
        std::cerr << "Need a command.\n";
    } else if (std::strcmp(cmd, "zcat") == 0) {
        while (const char* p = *++argv) {
            try {
                Ptr<Stream> f = Stream::openFile(p, Stream::C_READ);
                ZipStream zs(*f, true);
                cat(zs);
            }
            catch (FileProblemException& p) {
                std::cout << p.getFileName() << ": " << p.what() << "\n";
            }
        }
    } else if (std::strcmp(cmd, "ls") == 0) {
        while (const char* p = *++argv) {
            try {
                // Read directory
                Ptr<Stream> f = Stream::openFile(p, Stream::C_READ);
                Ptr<ZipFile> zf(ZipFile::open(f, ZipFile::KeepPaths));
                Ptr<Enumeration<Ptr<DirectoryEntry> > > entries(zf->getEntries());
                while (entries->hasMoreElements()) {
                    Ptr<DirectoryEntry> entry = entries->getNextElement();
                    std::cout << "entry '" << entry->getName() << "', " << entry->getSize() << " bytes\n";
                }
            }
            catch (FileProblemException& p) {
                std::cout << p.getFileName() << ": " << p.what() << "\n";
            }
        }
    } else if (std::strcmp(cmd, "get") == 0) {
        const char* zname = *++argv;
        if (!zname) {
            std::cerr << "Need a zip file name.\n";
        } else {
            try {
                Ptr<Stream> f = Stream::openFile(zname, Stream::C_READ);
                Ptr<ZipFile> zf(ZipFile::open(f, ZipFile::KeepPaths));
                while (const char* p = *++argv) {
                    Ptr<Stream> zff = zf->openFile(p, Stream::C_READ);
                    cat(*zff);
                }
            }
            catch (FileProblemException& p) {
                std::cout << p.getFileName() << ": " << p.what() << "\n";
            }
        }
    } else {
        std::cerr << "Unknown command.\n";
    }
}
