/**
  *  \file tools/c2dumpsimple.cc
  *
  *  (c) 2001-2015 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "tools/c2dump.h"
#include "io/streams.h"
#include "cpluslib/format.h"

void
parseShipFile(Stream& s, DumpOutputReceiver& r)
{
    // Read file into memory
    Buffer<char> buf;
    s.readBuffer(buf);
    DumpBlockInput bi(buf.ptr(), buf.getSize());

    // Parse it
    if (bi.getRemainingSize() >= 2) {
        r.startRecord("Header");
        r.addField("Count", bi.readWord());
        r.endRecord();
    }

    int n = 0;
    while (bi.getRemainingSize() >= 107) {
        ++n;
        r.startRecord(format("Ship(#%d)", n));
        r.addField("Ship_Id",              bi.readWord());
        r.addField("Owner",                bi.readWord());
        r.addField("FCode",                bi.readString(3));
        r.addField("Speed",                bi.readWord());
        r.addField("Waypoint_delta",       bi.readCoordinate());
        r.addField("Location",             bi.readCoordinate());
        r.addField("Engine_type",          bi.readWord());
        r.addField("Hull_type",            bi.readWord());
        r.addField("Beam_type",            bi.readWord());
        r.addField("Beam_count",           bi.readWord());
        r.addField("Bay_count",            bi.readWord());
        r.addField("Tube_type",            bi.readWord());
        r.addField("Ammo",                 bi.readWord());
        r.addField("Tube_count",           bi.readWord());
        r.addField("Mission",              bi.readWord());
        r.addField("Primary_enemy",        bi.readWord());
        r.addField("Tow_Id",               bi.readWord());
        r.addField("Damage",               bi.readWord());
        r.addField("Crew",                 bi.readWord());
        r.addField("Cargo_Colonist_clans", bi.readWord());
        r.addField("Name",                 bi.readString(20));
        r.addField("Cargo_Neutronium",     bi.readWord());
        r.addField("Cargo_Tritanium",      bi.readWord());
        r.addField("Cargo_Duranium",       bi.readWord());
        r.addField("Cargo_Molybdenum",     bi.readWord());
        r.addField("Cargo_Supplies",       bi.readWord());
        r.addField("Unload_Neutronium",    bi.readWord());
        r.addField("Unload_Tritanium",     bi.readWord());
        r.addField("Unload_Duranium",      bi.readWord());
        r.addField("Unload_Molybdenum",    bi.readWord());
        r.addField("Unload_Colonists",     bi.readWord());
        r.addField("Unload_Supplies",      bi.readWord());
        r.addField("Unload_Id",            bi.readWord());
        r.addField("Transfer_Neutronium",  bi.readWord());
        r.addField("Transfer_Tritanium",   bi.readWord());
        r.addField("Transfer_Duranium",    bi.readWord());
        r.addField("Transfer_Molybdenum",  bi.readWord());
        r.addField("Transfer_Colonists",   bi.readWord());
        r.addField("Transfer_Supplies",    bi.readWord());
        r.addField("Transfer_Id",          bi.readWord());
        r.addField("Intercept_Id",         bi.readWord());
        r.addField("Cargo_Money",          bi.readWord());
        r.endRecord();
    }

    if (bi.getRemainingSize() >= 10) {
        r.startRecord("Trailer");
        r.addUnparsedData(bi.readUnparsed(10));
        r.endRecord();
    }

    dumpRemainder(bi, r);
}

void
parsePDataFile(Stream& s, DumpOutputReceiver& r)
{
    // Read file into memory
    Buffer<char> buf;
    s.readBuffer(buf);
    DumpBlockInput bi(buf.ptr(), buf.getSize());

    // Parse it
    if (bi.getRemainingSize() >= 2) {
        r.startRecord("Header");
        r.addField("Count", bi.readWord());
        r.endRecord();
    }

    int n = 0;
    while (bi.getRemainingSize() >= 85) {
        ++n;
        r.startRecord(format("Planet(#%d)", n));
        r.addField("Owner",              bi.readWord());
        r.addField("Planet_Id",          bi.readWord());
        r.addField("FCode",              bi.readString(3));
        r.addField("Mines",              bi.readWord());
        r.addField("Factories",          bi.readWord());
        r.addField("Defense",            bi.readWord());
        r.addField("Mined_Neutronium",   bi.readLong());
        r.addField("Mined_Tritanium",    bi.readLong());
        r.addField("Mined_Duraniumm",    bi.readLong());
        r.addField("Mined_Molybdenum",   bi.readLong());
        r.addField("Colonist_clans",     bi.readLong());
        r.addField("Supplies",           bi.readLong());
        r.addField("Money",              bi.readLong());
        r.addField("Ground_Neutronium",  bi.readLong());
        r.addField("Ground_Tritanium",   bi.readLong());
        r.addField("Ground_Duraniumm",   bi.readLong());
        r.addField("Ground_Molybdenum",  bi.readLong());
        r.addField("Density_Neutronium", bi.readWord());
        r.addField("Density_Tritanium",  bi.readWord());
        r.addField("Density_Duranium",   bi.readWord());
        r.addField("Density_Molybdenum", bi.readWord());
        r.addField("Colonist_tax",       bi.readWord());
        r.addField("Native_tax",         bi.readWord());
        r.addField("Colonist_happiness", bi.readWord());
        r.addField("Native_happiness",   bi.readWord());
        r.addField("Native_government",  bi.readWord());
        r.addField("Native_clans",       bi.readLong());
        r.addField("Native_race",        bi.readWord());
        r.addField("Temperature_code",   bi.readWord());
        r.addField("Build_base_flag",    bi.readWord());
        r.endRecord();
    }

    if (bi.getRemainingSize() >= 10) {
        r.startRecord("Trailer");
        r.addUnparsedData(bi.readUnparsed(10));
        r.endRecord();
    }

    while (bi.getRemainingSize() > 0) {
        r.addUnparsedData(bi.readUnparsed(16));
    }
}

void
parseBDataFile(Stream& s, DumpOutputReceiver& r)
{
    // Read file into memory
    Buffer<char> buf;
    s.readBuffer(buf);
    DumpBlockInput bi(buf.ptr(), buf.getSize());

    // Parse it
    if (bi.getRemainingSize() >= 2) {
        r.startRecord("Header");
        r.addField("Count", bi.readWord());
        r.endRecord();
    }

    int n = 0;
    while (bi.getRemainingSize() >= 85) {
        ++n;
        r.startRecord(format("Starbase(#%d)", n));
        r.addField("Planet_Id",         bi.readWord());
        r.addField("Owner",             bi.readWord());
        r.addField("Base_defense",      bi.readWord());
        r.addField("Damage",            bi.readWord());
        r.addField("Engine_tech",       bi.readWord());
        r.addField("Hull_tech",         bi.readWord());
        r.addField("Beam_tech",         bi.readWord());
        r.addField("Torpedo_tech",      bi.readWord());
        for (int i = 1; i <= 9; ++i)
            r.addField(format("Engine_storage(%d)", i), bi.readWord());
        for (int i = 1; i <= 20; ++i)
            r.addField(format("Hull_storage(%d)", i), bi.readWord());
        for (int i = 1; i <= 10; ++i)
            r.addField(format("Beam_storage(%d)", i), bi.readWord());
        for (int i = 1; i <= 10; ++i)
            r.addField(format("Tube_storage(%d)", i), bi.readWord());
        for (int i = 1; i <= 10; ++i)
            r.addField(format("Torpedo_storage(%d)", i), bi.readWord());

        r.addField("Fighters",          bi.readWord());
        r.addField("Shipyard_Id",       bi.readWord());
        r.addField("Shipyard_Order",    bi.readWord());
        r.addField("Mission",           bi.readWord());
        r.addField("Build_hull_slot",   bi.readWord());
        r.addField("Build_engine_type", bi.readWord());
        r.addField("Build_beam_type",   bi.readWord());
        r.addField("Build_beam_count",  bi.readWord());
        r.addField("Build_tube_type",   bi.readWord());
        r.addField("Build_tube_count",  bi.readWord());
        r.addField("Unused",            bi.readWord());
        r.endRecord();
    }    

    if (bi.getRemainingSize() >= 10) {
        r.startRecord("Trailer");
        r.addUnparsedData(bi.readUnparsed(10));
        r.endRecord();
    } else if (bi.getRemainingSize() >= 2) {
        r.startRecord("Trailer");
        r.addField("Turn_number", bi.readWord());
        r.endRecord();
    }

    dumpRemainder(bi, r);
}

void
parseGenFile(Stream& s, DumpOutputReceiver& r)
{
    // Read file into memory
    Buffer<char> buf;
    s.readBuffer(buf);
    DumpBlockInput bi(buf.ptr(), buf.getSize());

    r.startRecord("Timestamp");
    r.addField("Timestamp", bi.readString(18));
    r.endRecord();

    for (int i = 1; i <= 11; ++i) {
        r.startRecord(format("Score(%d)", i));
        r.addField("Num_planets",       bi.readWord());
        r.addField("Num_capital_ships", bi.readWord());
        r.addField("Num_freighters",    bi.readWord());
        r.addField("Num_starbases",     bi.readWord());
        r.endRecord();
    }

    r.startRecord("Identification");
    r.addField("Owner",              bi.readWord());
    r.addField("Password_1",         bi.readUnparsed(10));
    r.addField("Password_2",         bi.readUnparsed(10));
    r.addField("Unused",             bi.readUnparsed(1));
    r.addField("Ship_checksum",      bi.readLong());
    r.addField("Planet_checksum",    bi.readLong());
    r.addField("Starbase_checksum",  bi.readLong());
    r.addField("New_password_flag",  bi.readWord());
    r.addField("New_password",       bi.readUnparsed(10));
    r.addField("Turn_number",        bi.readWord());
    r.addField("Timestamp_checksum", bi.readWord());
    r.endRecord();

    dumpRemainder(bi, r);
}

static void
parseVcrObject(DumpOutputReceiver& r, DumpBlockInput& bi)
{
    r.addField("Name",              bi.readString(20));
    r.addField("Damage",            bi.readWord());
    r.addField("Crew",              bi.readWord());
    r.addField("Id",                bi.readWord());
    r.addField("Owner",             bi.readByte());
    r.addField("Owner_race",        bi.readByte());
    r.addField("Picture",           bi.readByte());
    r.addField("Hull",              bi.readByte());
    r.addField("Beam_type",         bi.readWord());
    r.addField("Beam_count",        bi.readByte());
    r.addField("Experience_level",  bi.readByte());
    r.addField("Bay_count",         bi.readWord());
    r.addField("Tube_type",         bi.readWord());
    r.addField("Ammo",              bi.readWord());
    r.addField("Tube_count",        bi.readWord());
}

void
parseVcrFile(Stream& s, DumpOutputReceiver& r)
{
    // Read file into memory
    Buffer<char> buf;
    s.readBuffer(buf);
    DumpBlockInput bi(buf.ptr(), buf.getSize());

    // Parse it
    if (bi.getRemainingSize() >= 2) {
        r.startRecord("Header");
        r.addField("Count", bi.readWord());
        r.endRecord();
    }

    int n = 0;
    while (bi.getRemainingSize() >= 100) {
        ++n;
        r.startRecord(format("VCR(#%d)", n));
        r.addField("Seed",                 bi.readWord());
        r.addField("Signature",            bi.readWord());
        r.addField("Temp_or_Capabilities", bi.readWord());
        r.addField("Type",                 bi.readWord());
        r.addField("Left_mass",            bi.readWord());
        r.addField("Right_mass",           bi.readWord());
        r.startRecord("Left_object");
        parseVcrObject(r, bi);
        r.endRecord();
        r.startRecord("Right_object");
        parseVcrObject(r, bi);
        r.endRecord();
        r.addField("Left_shield",          bi.readWord());
        r.addField("Right_shield",         bi.readWord());
        r.endRecord();
    }    

    if (bi.getRemainingSize() >= 10) {
        r.startRecord("Trailer");
        r.addUnparsedData(bi.readUnparsed(10));
        r.endRecord();
    }

    dumpRemainder(bi, r);
}

void
parseTargetFile(Stream& s, DumpOutputReceiver& r)
{
    // Read file into memory
    Buffer<char> buf;
    s.readBuffer(buf);
    DumpBlockInput bi(buf.ptr(), buf.getSize());

    // Parse it
    if (bi.getRemainingSize() >= 2) {
        r.startRecord("Header");
        r.addField("Count", bi.readWord());
        r.endRecord();
    }

    int n = 0;
    while (bi.getRemainingSize() >= 34) {
        ++n;
        r.startRecord(format("Target(#%d)", n));
        r.addField("Ship_Id",              bi.readWord());
        r.addField("Owner",                bi.readWord());
        r.addField("Speed",                bi.readWord());
        r.addField("Location",             bi.readCoordinate());
        r.addField("Hull_type",            bi.readWord());
        r.addField("Heading",              bi.readWord());
        r.addField("Name",                 bi.readString(20));
        r.endRecord();
    }

    if (bi.getRemainingSize() >= 10) {
        r.startRecord("Trailer");
        r.addUnparsedData(bi.readUnparsed(10));
        r.endRecord();
    }

    dumpRemainder(bi, r);
}
